
/*
     _________ __________  _____ ___      ________    __________    ________
    /   _____/ \______   \ \__  |   |     \_      \   \______   \  /  _____/
    \_____  \   |     ___/  /   |   |      /   |   \   |       _/ /   \  ___
    /        \  |    |      \____   |     /    |    \  |    |   \ \    \_\  \
   /_______  /  |____|      / ______|  /\ \_______  /  |____|_  /  \______  /
  ==========\/==============\/=========\/=========\/==========\/==========\/==
  V0.01 31/AUG/2007 (C) Copyright 2007-2007 Scott D. Yelich ALL RIGHTS RESERVED
 
  LICENSE:  Creative Commons Attribution 3.0 License.
  SEE:      http://creativecommons.org/licenses/by/3.0/

  Fri Aug 31 23:33:35 EDT 2007, v0.01 sdy
 
*/

/*
    This file is part of CGI
*/

#include "Cgi.h"

Cgi::Cgi()
{
}

Cgi::~Cgi()
{
}

void
Cgi::getGetData()
{
  char * qs = getenv("QUERY_STRING");
  if (qs) {
    buffer.clear();
    buffer.append(qs);
  }
}

void
Cgi::getPostData()
{
  std::stringstream ss;
  ss << std::cin.rdbuf();  //  CONTENT_LENGTH?
  buffer = ss.str();
}

void
Cgi::mapkvp(std::string const & kvp)
{
  std::string::size_type pos = 0;
  if (( pos = kvp.find("=", 0)) != std::string::npos ) {
    data[urldecode(kvp.substr(0, pos))] = urldecode(kvp.substr(pos+1, kvp.size()));
  }
}

char
Cgi::hex2char(std::string & hex)
{
  // no error checking
  return ((hex[0] >= 'A' ? (((hex[0]&0x5f)-'A')+10) : (hex[0]-'0'))*16 +
          (hex[1] >= 'A' ? (((hex[1]&0x5f)-'A')+10) : (hex[1]-'0'))     ) & 255;
}

std::string
Cgi::urldecode(std::string istr)
{
  std::string hexcs;
  std::string dhchar;
  std::string::size_type pos = 0;
  //  '+' --> ' '
  while ( (pos = istr.find("+", pos)) != std::string::npos ) {
    istr.replace(pos, 1, " ");
  }
  //  "%xy" --> 'C'
  while ( (pos = istr.find("%", pos)) != std::string::npos ) {
    hexcs = istr.substr(pos+1, pos+2);
    dhchar = hex2char(hexcs);
    istr.replace(pos, 3, dhchar);
    ++pos; // skip newly converted char 
  }
  return istr;
}

void
Cgi::parse()
{
  //  parse to map "key1=val1&key2=val2" --> map: [key1]="val1", [key2]="val2"
  //  note:  this isn't a general CGI parser -- so no multivalues or keys 
  //         without values such as "sel=one&sel=two&high&up" ...
  std::string::size_type cur = 0;
  std::string::size_type pos = 0;
  std::string::size_type posx = 0;
  while ( (pos = buffer.find("&", pos)) != std::string::npos ) {
    mapkvp(buffer.substr(cur, pos-cur));
    pos++;
    cur = pos;
  }  
  pos = buffer.size();
  if ( pos-cur != 0 ) {
    if ( ( posx = buffer.substr(cur, pos-cur).find("=", 0)) != std::string::npos) {
      mapkvp(buffer.substr(cur, pos-cur));
    }
  }
}

void
Cgi::pmap()
{
//  for ( strstrMap::const_iterator iter = data.begin(); iter != data.end(); ++iter ) {
//    std::cout << iter->first << "=" << iter->second << std::endl;
//  }
  strstrMap::iterator iter = data.begin();
  strstrMap::iterator end = data.end();
  while ( iter != end ) {
    std::cout << iter->first << "=" << iter->second << std::endl;
    iter++;
  }
}

void
Cgi::process()
{
  char * rm = getenv("REQUEST_METHOD");
  if (rm && !strcmp("GET", rm)) {
    getGetData();
  } else {
    getPostData();
  }
  parse();
//  pmap();
}

std::string
Cgi::getVal(std::string const & key)
{
  strstrMap::iterator iter;
  std::string val;
  iter = data.find(key);
  if ( iter != data.end() ) {
    val = data[key];
  }
  return val;
}
